﻿/*---------------------------------------------------------------------------*
  Project:  Horizon
  File:     WaveFileReader.h

  Copyright (C)2009-2013 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Rev: 50863 $
 *---------------------------------------------------------------------------*/

#ifndef NN_SAMPLE_DEMOS_SND_COMMON_WAVE_FILE_READER_H_
#define NN_SAMPLE_DEMOS_SND_COMMON_WAVE_FILE_READER_H_

#include <nn/fs.h>

class WaveFileReader
{
private:
    nn::fs::FileReader m_Reader;
    s64 m_DataSize;
    s64 m_DataOffset;
    s64 m_ReadPosition;
    s32 m_ChannelCount;
    s32 m_SampleRate;
    s32 m_QuantumBits;
    NN_PADDING4;
private:
    void ReadInfo();
public:
    /*!
        @brief Constructor. Does not open files.
     */
    WaveFileReader();
    /*!
        @brief      Constructor that opens the specified file(s).
        @param[in]  pPathName:   File path.
     */
    WaveFileReader(const char* pPathName);
    /*!
        @brief Destructor.
                    If the Finalize function has not been called, this will automatically call it.
     */
    ~WaveFileReader();
    /*!
        @brief      Opens the specified file(s).
                    Does nothing if the file is already open.
        @param[in]  pPathName:   File path.
     */
    void Initialize(const char* pPathName);
    /*!
        @brief      Closes the file.
                    Does nothing if the file is already closed.
        @param[in]  pPathName:   File path.
     */
    void Finalize();
    /*!
        @brief      Gets the size of waveform data in bytes.
                    Returns 0 if the file is not open.
        @return     Size of waveform data.
     */
    s64 GetDataSize() const { return m_DataSize; }
    /*!
        @brief      Get the number of channels.
                    Returns 0 if the file is not open.
        @return     Number of channels
     */
    s32 GetChannelCount() const { return m_ChannelCount; }
    /*!
        @brief      Gets the sampling rate.
                    Returns 0 if the file is not open.
        @return     Sampling rate.
     */
    s32 GetSampleRate() const { return m_SampleRate; }
    /*!
        @brief      Gets the number of quantization bits.
                    Returns 0 if the file is not open.
        @return     Number of quantization bits.
     */
    s32 GetQuantumBits() const { return m_QuantumBits; }
    /*!
        @brief      Gets waveform data.
        @param[out] pData:       Buffer that stores waveform data.
        @param[in]  size:        Size of waveform data in bytes.
        @return     Returns the size (in bytes) of the waveform data that was actually obtained.
     */
    s64 ReadData(void* pData, s64 size);
};

#endif  // NN_SAMPLE_DEMOS_SND_COMMON_WAVE_FILE_READER_H_
